/*
 * BEGIN LICENSE
 * This file is in the public domain
 * END LICENSE
 */
using Dee;
using Gee;
using Unity.Place;
using Config;

namespace Unity.SamplePlace {
  
  /* Absolute path to custom icons */
  const string ICON_PATH = Config.DATADIR + "/icons/unity-icon-theme/places/svg/";
  
  /* DBus name for the place. Must match out .place file */
  const string BUS_NAME = "com.canonical.Unity.SamplePlace";

  /**
   * The Daemon class implements all of the logic for the place.
   * It implements the Unity.Place.Activation interface in order to
   * do custom activation of the URI and Mime patterns defined in the
   * .place file.
   *
   */
  public class Daemon : GLib.Object, Unity.Place.Activation
  {
    private Unity.Place.Controller control;
    private Unity.Place.EntryInfo place_entry;

    construct
    {
      /* Create all the models we need to share with the Unity shell.
       * The model column schemas must align with what we defined in
       * schemas.vala */
      var sections_model = new Dee.SharedModel(BUS_NAME + ".SectionsModel");
      sections_model.set_schema ("s", "s");

      var groups_model = new Dee.SharedModel(BUS_NAME + ".GroupsModel");
      groups_model.set_schema ("s", "s", "s");

      var global_groups_model = new Dee.SharedModel(BUS_NAME + ".GlobalGroupsModel");
      global_groups_model.set_schema ("s", "s", "s");

      var results_model = new Dee.SharedModel(BUS_NAME + ".ResultsModel");
      results_model.set_schema ("s", "s", "u", "s", "s", "s");

      var global_results_model = new Dee.SharedModel(BUS_NAME + ".GlobalResultsModel");
      global_results_model.set_schema ("s", "s", "u", "s", "s", "s");
      
      /* Create an EntryInfo. The properties of the EntryInfo is automatically
       * mapped back and forth over the bus, and removes most DBusisms from
       * the place daemon code.
       *
       * Object path of the EntryInfo must match the one defined in the
       * the place entry in the place .place file
       */
      place_entry = new EntryInfo ("/com/canonical/unity/sampleplace/stuff");
      place_entry.sections_model = sections_model;
      place_entry.entry_renderer_info.groups_model = groups_model;
      place_entry.entry_renderer_info.results_model = results_model;
      place_entry.global_renderer_info.groups_model = global_groups_model;
      place_entry.global_renderer_info.results_model = global_results_model;

      populate_sections ();
      populate_groups ();

      /* Unity preloads the icon defined in the .place file even before the
       * place daemon is running - when we come up it will ratify the icon */
      place_entry.icon = @"$(Config.PREFIX)/share/unity/sample-place-icon.png";

      /* Listen for section changes and start updating our results model
       * accordingly */
      place_entry.notify["active-section"].connect (update_entry_results_model);

      /* Listen for changes to the place entry search */
      place_entry.notify["active-search"].connect (update_entry_results_model);

      /* Listen for changes to the global search */
      place_entry.notify["active-global-search"].connect (update_global_results_model);
      
      /* Listen for when the place is hidden/shown by the Unity Shell */
      place_entry.notify["active"].connect (
        (obj, pspec) => {
          debug (@"Activated: $(place_entry.active)");
        }
      );          

      /* The last thing we do is export the controller. Once that is up,
       * clients will expect the Dee.SharedModels to work.
       * The 'place' EntryInfo is exported for Unity to show by adding it to
       * the controller. You can add more than one EntryInfo here if you have
       * more than one place entry to export from the same place daemon */
      control = new Unity.Place.Controller ("/com/canonical/unity/sampleplace");
      control.add_entry (place_entry);
      
      /* Since the Daemon class implements the Unity.Place.Activation interface
       * we can override the default activation handler on the controller.
       * We need to do that to properly handle the activation patternts we
       * registered in the .place file  */
      control.activation = this;

      /* Export the controller on the bus. Unity can see us past this point */
      try {
        control.export ();
      } catch (IOError error) {
        critical ("Failed to export DBus service for '%s': %s",
                  control.dbus_path, error.message);
      }
    }

    private void populate_sections ()
    {
      var sections = place_entry.sections_model;

      if (sections.get_n_rows() != 0)
        {
          critical ("The sections model should be empty before initial population");
          sections.clear ();
        }

      /* The row offsets should match those from the Section enum */
      sections.append (_("All Stuff"), "");
      sections.append (_("My Stuff"), "");
      sections.append (_("Your Stuff"), "");
    }

    private void populate_groups ()
    {
      var groups = place_entry.entry_renderer_info.groups_model;

      if (groups.get_n_rows() != 0)
        {
          critical ("The groups model should be empty before initial population");
          groups.clear ();
        }

      /* The row offsets should match those from the Group enum */
      groups.append ("UnityDefaultRenderer", _("Group One"), ICON_PATH + "group-mostused.svg");
      groups.append ("UnityDefaultRenderer", _("Group Two"), ICON_PATH + "group-recent.svg");
    }
    
    private void update_entry_results_model ()
    {
      /* If we're not active just ignore anything Unity tells us to do
       * it's not gonna show on the screen anyway */
      if (!place_entry.active)
        return;
                    
      var _results_model = place_entry.entry_renderer_info.results_model;
      var _groups_model = place_entry.entry_renderer_info.groups_model;
      var _search = place_entry.active_search;
      var _section = (Section)place_entry.active_section;          
       
      update_results_model (_results_model, _groups_model,
                            _search, _section);
    }
    
    private void update_global_results_model ()
    {
      /* Unlike update_entry_model() the global model may be updated
       * even though we are not active */
    
      var _results_model = place_entry.global_renderer_info.results_model;
      var _groups_model = place_entry.global_renderer_info.groups_model;
      var _search = place_entry.active_global_search;
      var _section = Section.ALL_STUFF; /* We have no active section in global mode */
       
      update_results_model (_results_model, _groups_model,
                            _search, _section);
    }

    /* Generic method to update a results model. We do it like this to minimize
     * code dup between updating the global- and the entry results model */
    private void update_results_model (Dee.Model results_model, Dee.Model groups_model,
                                       Search? search, Section section)
    {
      debug ("Rebuilding results model");
      results_model.clear ();
      
      var icon = g_content_type_get_icon ("inode/directory");
      results_model.append ("file:///home",
                            icon.to_string (), /* Must be a serialized GIcon */
                            Group.GROUP_ONE,
                            "inode/directory",
                            "/home folder",
                            "Shared user home dir");
      
      icon = g_content_type_get_icon ("image/png");
      results_model.append ("file:///usr/share/icons/hicolor/48x48/apps/evolution-mail.png",
                            icon.to_string (),
                            Group.GROUP_TWO,
                            "image/png",
                            "Evolution icon",
                            "This is a png with the Evolution icon");
    }

    /**
     * Override of the default activation handler. Unity will ask the
     * place daemon for activation of the URI pattern and mime type pattern
     * defined in the .place file.
     *
     * This method should return a member of the enumeration
     * Unity.Place.ActivationStatus:
     *
     * - ActivationStatus.ACTIVATED_HIDE_DASH
     * - ActivationStatus.ACTIVATED_SHOW_DASH
     * - ActivationStatus.NOT_ACTIVATED
     *
     */
    public async uint32 activate (string uri)
    {
      debug ("Requested activation of: %s", uri);
      return ActivationStatus.NOT_ACTIVATED;
    }
    
  } /* End Daemon class */

} /* End Unity.SamplePlace namespace */
